@@ -1,4 +1,4 @@
-/* $Id: SUPDrvIOC.h 139685 2020-07-31 19:40:21Z bird $ */
+/* $Id: SUPDrvIOC.h 140899 2020-10-13 13:14:24Z bird $ */
 /** @file
  * VirtualBox Support Driver - IOCtl definitions.
  */
@@ -222,7 +222,7 @@ typedef SUPREQHDR *PSUPREQHDR;
  * @todo Pending work on next major version change:
  *          - Nothing.
  */
-#define SUPDRV_IOC_VERSION                              0x002e0000
+#define SUPDRV_IOC_VERSION                              0x00300000
 
 /** SUP_IOCTL_COOKIE. */
 typedef struct SUPCOOKIE
@@ -480,6 +480,8 @@ typedef struct SUPLDRLOAD
             uint32_t        cSegments;
             /** Size of image data in achImage. */
             uint32_t        cbImageWithEverything;
+            /** Flags (SUPLDRLOAD_F_XXX). */
+            uint32_t        fFlags;
             /** The image data. */
             uint8_t         abImage[1];
         } In;
@@ -497,6 +499,10 @@ typedef struct SUPLDRLOAD
  * present on SUP_IOCTL_LDR_LOAD failure.
  * @remarks The value is choosen to be an unlikely init and term address. */
 #define SUPLDRLOAD_ERROR_MAGIC      UINT64_C(0xabcdefef0feddcb9)
+/** The module depends on VMMR0. */
+#define SUPLDRLOAD_F_DEP_VMMR0      RT_BIT_32(0)
+/** Valid flag mask. */
+#define SUPLDRLOAD_F_VALID_MASK     UINT32_C(0x00000001)
 /** @} */
 
 
